@@ -42,7 +42,7 @@
 #include "tio/log.h"
 #include "tio/error.h"
 
-static struct termios tio, new_stdout, old_stdout, old_tio;
+static struct termios tio, tio_old, stdout_new, stdout_old, stdin_new, stdin_old;
 static unsigned long rx_total = 0, tx_total = 0;
 static bool connected = false;
 static bool tainted = false;
@@ -112,6 +112,8 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                 tio_printf(" Stopbits: %d", option.stopbits);
                 tio_printf(" Parity: %s", option.parity);
                 tio_printf(" Output delay: %d", option.output_delay);
+                if (option.map[0] != 0)
+                    tio_printf(" Map flags: %s", option.map);
                 if (option.log)
                     tio_printf(" Log file: %s", option.log_filename);
                 break;
@@ -161,30 +163,70 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
     }
 }
 
+void stdin_configure(void)
+{
+    int status;
+
+    /* Save current stdin settings */
+    if (tcgetattr(STDIN_FILENO, &stdin_old) < 0)
+    {
+        error_printf("Saving current stdin settings failed");
+        exit(EXIT_FAILURE);
+    }
+
+    /* Prepare new stdin settings */
+    memcpy(&stdin_new, &stdin_old, sizeof(stdin_old));
+
+    /* Reconfigure stdin (RAW configuration) */
+    stdin_new.c_iflag &= ~(ICRNL); // Do not translate CR -> NL on input
+    stdin_new.c_oflag &= ~(OPOST);
+    stdin_new.c_lflag &= ~(ECHO|ICANON|ISIG|ECHOE|ECHOK|ECHONL);
+
+    /* Control characters */
+    stdin_new.c_cc[VTIME] = 0; /* Inter-character timer unused */
+    stdin_new.c_cc[VMIN]  = 1; /* Blocking read until 1 character received */
+
+    /* Activate new stdin settings */
+    status = tcsetattr(STDIN_FILENO, TCSANOW, &stdin_new);
+    if (status == -1)
+    {
+        error_printf("Could not apply new stdin settings (%s)", strerror(errno));
+        exit(EXIT_FAILURE);
+    }
+
+    /* Make sure we restore old stdin settings on exit */
+    atexit(&stdin_restore);
+}
+
+void stdin_restore(void)
+{
+    tcsetattr(STDIN_FILENO, TCSANOW, &stdin_old);
+}
+
 void stdout_configure(void)
 {
     int status;
 
     /* Save current stdout settings */
-    if (tcgetattr(STDOUT_FILENO, &old_stdout) < 0)
+    if (tcgetattr(STDOUT_FILENO, &stdout_old) < 0)
     {
         error_printf("Saving current stdio settings failed");
         exit(EXIT_FAILURE);
     }
 
     /* Prepare new stdout settings */
-    memcpy(&new_stdout, &old_stdout, sizeof(old_stdout));
+    memcpy(&stdout_new, &stdout_old, sizeof(stdout_old));
 
     /* Reconfigure stdout (RAW configuration) */
-    new_stdout.c_oflag &= ~(OPOST);
-    new_stdout.c_lflag &= ~(ECHO|ICANON|ISIG|ECHOE|ECHOK|ECHONL);
+    stdout_new.c_oflag &= ~(OPOST);
+    stdout_new.c_lflag &= ~(ECHO|ICANON|ISIG|ECHOE|ECHOK|ECHONL);
 
     /* Control characters */
-    new_stdout.c_cc[VTIME] = 0; /* Inter-character timer unused */
-    new_stdout.c_cc[VMIN]  = 1; /* Blocking read until 1 character received */
+    stdout_new.c_cc[VTIME] = 0; /* Inter-character timer unused */
+    stdout_new.c_cc[VMIN]  = 1; /* Blocking read until 1 character received */
 
     /* Activate new stdout settings */
-    status = tcsetattr(STDOUT_FILENO, TCSANOW, &new_stdout);
+    status = tcsetattr(STDOUT_FILENO, TCSANOW, &stdout_new);
     if (status == -1)
     {
         error_printf("Could not apply new stdout settings (%s)", strerror(errno));
@@ -204,11 +246,14 @@ void stdout_configure(void)
 
 void stdout_restore(void)
 {
-    tcsetattr(STDOUT_FILENO, TCSANOW, &old_stdout);
+    tcsetattr(STDOUT_FILENO, TCSANOW, &stdout_old);
 }
 
 void tty_configure(void)
 {
+    bool token_found = true;
+    char *token = NULL;
+    char *buffer;
     int status;
     speed_t baudrate;
 
@@ -326,6 +371,48 @@ void tty_configure(void)
         error_printf("Invalid parity");
         exit(EXIT_FAILURE);
     }
+
+    /* Control, input, output, local modes for tty device */
+    tio.c_cflag |= CLOCAL | CREAD;
+    tio.c_oflag = 0;
+    tio.c_lflag = 0;
+
+    /* Control characters */
+    tio.c_cc[VTIME] = 0; // Inter-character timer unused
+    tio.c_cc[VMIN]  = 1; // Blocking read until 1 character received
+
+    /* Configure any specified input or output mappings */
+
+    buffer = strdup(option.map);
+    while (token_found == true)
+    {
+        if (token == NULL)
+            token = strtok(buffer,",");
+        else
+            token = strtok(NULL, ",");
+
+        if (token != NULL)
+        {
+            if (strcmp(token,"INLCR") == 0)
+                tio.c_iflag |= INLCR;
+            else if (strcmp(token,"IGNCR") == 0)
+                tio.c_iflag |= IGNCR;
+            else if (strcmp(token,"ICRNL") == 0)
+                tio.c_iflag |= ICRNL;
+            else if (strcmp(token,"ONLCR") == 0)
+                tio.c_oflag |= ONLCR;
+            else if (strcmp(token,"OCRNL") == 0)
+                tio.c_oflag |= OCRNL;
+            else
+            {
+                printf("Error: Unknown mapping flag %s\n", token);
+                exit(EXIT_FAILURE);
+            }
+        }
+        else
+            token_found = false;
+    }
+    free(buffer);
 }
 
 void tty_wait_for_device(void)
@@ -399,7 +486,7 @@ void tty_disconnect(void)
 
 void tty_restore(void)
 {
-    tcsetattr(fd, TCSANOW, &old_tio);
+    tcsetattr(fd, TCSANOW, &tio_old);
 
     if (connected)
         tty_disconnect();
@@ -446,7 +533,7 @@ int tty_connect(void)
     tainted = false;
 
     /* Save current port settings */
-    if (tcgetattr(fd, &old_tio) < 0)
+    if (tcgetattr(fd, &tio_old) < 0)
         goto error_tcgetattr;
 
     /* Make sure we restore tty settings on exit */
@@ -456,15 +543,6 @@ int tty_connect(void)
         first = false;
     }
 
-    /* Control, input, output, local modes for tty device */
-    tio.c_cflag |= CLOCAL | CREAD;
-    tio.c_oflag = 0;
-    tio.c_lflag = 0;
-
-    /* Control characters */
-    tio.c_cc[VTIME] = 0; /* Inter-character timer unused */
-    tio.c_cc[VMIN]  = 1; /* Blocking read until 1 character received */
-
     /* Activate new port settings */
     status = tcsetattr(fd, TCSANOW, &tio);
     if (status == -1)
